from django import forms
from django.utils.translation import ugettext_lazy as _

from tagging.forms import TagField

from bookmarks.models import Bookmark, BookmarkInstance

class BookmarkInstanceForm(forms.ModelForm):
    
    url = forms.URLField(label = "URL", verify_exists=True, widget=forms.TextInput(attrs={"size": 40}))
    description = forms.CharField(max_length=100, widget=forms.TextInput(attrs={"size": 40}))
    redirect = forms.BooleanField(label="Redirect", required=False)
    tags = TagField(label="Tags", required=False)
    
    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(BookmarkInstanceForm, self).__init__(*args, **kwargs)
        # hack to order fields
        self.fields.keyOrder = ['url', 'description', 'note', 'tags', 'redirect']
    
    def clean(self):
        if 'url' not in self.cleaned_data:
            return
        if BookmarkInstance.on_site.filter(bookmark__url=self.cleaned_data['url'], user=self.user).count() > 0:
            raise forms.ValidationError(_("You have already bookmarked this link."))
        return self.cleaned_data
                
    def should_redirect(self):
        if self.cleaned_data["redirect"]:
            return True
        else:
            return False
    
    def save(self, commit=True):
        self.instance.url = self.cleaned_data['url']
        return super(BookmarkInstanceForm, self).save(commit)
    
    class Meta:
        model = BookmarkInstance
        #fields = ('url', 'description', 'note', 'redirect')



from django import template
from bookmarks.models import BookmarkInstance
from tagging.models import Tag

register = template.Library()

@register.inclusion_tag('bookmarks/tags.html')
def show_bookmarks_tags():
    """ Show a box with tags for all articles that belong to current site.
    """
    return {'bookmark_tags': Tag.objects.usage_for_queryset(queryset=BookmarkInstance.on_site.all(), counts=True, min_count=1)}


from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.conf import settings

from bookmarks.models import Bookmark, BookmarkInstance
from voting.models import Vote



class UserBookmarksTests(TestCase):
    fixtures = ['accountdata.json', 'bookmarks.json', 'sites.json']

    def checkAddBookmark(self, test_url = 'http://google.com/', test_description = 'test bookmark description', user=None):
        ' Check if new "add bookmark" page works. Can be used to check if new bookmark is added or if existing bookmark is saved. '

        all_bookmarks_url = reverse('all_bookmarks')

        url = reverse('add_bookmark')
        response = self.client.post(url, {'url': test_url, 'description' : test_description, 'tags': 'new-tag'}, follow=True)
        self.assertRedirects(response, all_bookmarks_url)
        self.assertContains(response, 'new-tag')

        bookmark_instance = BookmarkInstance.on_site.order_by('-id')[0]
        bookmark = Bookmark.on_site.all().order_by('-id')[0]

        # Check if it is the new bookmark and new instance is created.
        self.assertEquals(bookmark.url, test_url)
        self.assertEquals(bookmark.sites.all()[0].id, 1)
        self.assertEquals(bookmark.sites.all()[0].id, 1)
        self.assertEquals(bookmark_instance.description, test_description)
        self.assertEquals(bookmark_instance.user, user)
        self.assertEquals(bookmark_instance.bookmark, bookmark)

        # Check if the new bookmark has appeared on the page
        response = self.client.get(all_bookmarks_url)
        self.assertContains(response, test_url, 3, 200,)
        self.assertContains(response, test_description, 1, 200,)

        #Try to add existing bookmark.
        response = self.client.post(url, {'url': test_url, 'description' : test_description,}, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertFormError(response, 'bookmark_form', None, 'You have already bookmarked this link.', msg_prefix='')

        self.client.logout()


    # Tests if user can rate bookmarks
    def rateBookmark(self, direction, bookmark=None):
        ' Check if a bookmark can be voted correctly.'
        assert direction in ['up', 'down']

        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        if bookmark == None:
            bookmark = Bookmark.on_site.all()[0]

        # Get initial Score value.
        initial = Vote.objects.get_score(bookmark)
        initial_score = int(initial['score'])
        initial_num = int(initial['num_votes'])

        # Vote
        url = reverse('voting.views.vote_on_object', kwargs={'object_id' : bookmark.id, 'direction': direction, }, )
        response = self.client.post(url, **{'HTTP_X_REQUESTED_WITH': 'XMLHttpRequest'})
        self.assertContains(response, '"success": true',  1, 200)

        # Compare with new score value
        new = Vote.objects.get_score(bookmark)
        new_score = int(new['score'])
        new_num = int(new['num_votes'])
        if direction == 'up':
            self.assertEquals(new_score, initial_score+1)
        else:
            self.assertEquals(new_score, initial_score-1)
        self.assertEquals(new_num, initial_num+1)

        # Check if ajax response contains correct values
        self.assertContains(response, '"score": ' + str(new_score), 1, 200)
        self.assertContains(response, '"num_votes": ' + str(new_num), 1, 200)

        self.client.logout()



    # Tests if user can add new bookmark and rate it.
    def testUserBookmarkNew(self):
        ' Tests if a user can add bookmarks correctly. '
        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        url = reverse('add_bookmark')
        response = self.client.get(url,)
        self.assertEquals(response.status_code, 200)

        self.checkAddBookmark(user=user)

        self.client.logout()


    def testVoteUp(self):

        self.rateBookmark('up')

    def testVoteDown(self):

        self.rateBookmark('down')


    def testSaveBookmark(self):
        ' Tests if user can save bookmark. '
        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        bookmark = Bookmark.objects.all()[1]

        url = reverse('add_bookmark')
        response = self.client.post(url, kwargs={'url':bookmark.url, 'description':bookmark.description})
        self.assertEquals(response.status_code, 200)
        
        self.checkAddBookmark(test_url=bookmark.url, test_description=bookmark.description, user=user)

        self.client.logout()

    def testSiteSpecificBookmark(self):
        ' Tests site specific bookmarks. '
        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        # Deafault SITE_ID=1
        bookmarks = Bookmark.objects.all()
        bookmark_site1 = bookmarks.get(id=1)
        bookmark_site2 = bookmarks.get(id=2)

        url = reverse('all_bookmarks')
        response = self.client.get(url)
        self.assertContains(response, bookmark_site1.url)
        self.assertContains(response, bookmark_site1.description)
        self.assertNotContains(response, bookmark_site2.url)
        self.assertNotContains(response, bookmark_site2.description)

        self.assertContains(response, 'tag-site1')
        self.assertNotContains(response, 'tag-site2')

        settings.SITE_ID=2

        response = self.client.get(url)
        self.assertContains(response, bookmark_site2.url)
        self.assertContains(response, bookmark_site2.description)
        self.assertNotContains(response, bookmark_site1.url)
        self.assertNotContains(response, bookmark_site1.description)

        self.assertContains(response, 'tag-site2')
        self.assertNotContains(response, 'tag-site1')

        settings.SITE_ID=1
        self.client.logout()


from haystack.indexes import CharField, DateTimeField, RealTimeSearchIndex, Indexable

from models import Bookmark


class BookmarkIndex(RealTimeSearchIndex, Indexable):
    text = CharField(document=True, use_template=True)
    title = CharField(model_attr='description')
    author = CharField(model_attr='adder')
    pub_date = DateTimeField(model_attr='added')
    summary = CharField(model_attr='note')
    sites = CharField(model_attr='site_slugs')

    def index_queryset(self):
        """Used when the entire index for model is updated."""
        return self.get_model().objects.all()

    def get_model(self):
        return Bookmark



from setuptools import find_packages, setup
from os.path import dirname, join

from bookmarks import get_version

def fread(fname):
    return open(join(dirname(__file__), fname)).read()

setup(
    name='incuna-bookmarks',
    version=get_version(),
    description='A reusable Django app for bookmark management.',
    long_description=fread('README'),
    author='Incuna Ltd',
    author_email='admin@incuna.com',
    url='http://github.com/incuna/incuna-bookmarks',
    packages=find_packages(),
    include_package_data=True,
    package_dir={'bookmarks': 'bookmarks'},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)



from datetime import datetime
import urllib2

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from bookmarks.models import Bookmark, BookmarkInstance
from bookmarks.forms import BookmarkInstanceForm


def bookmarks(request, template_name="bookmarks/bookmarks.html"):
    bookmarks = Bookmark.on_site.all().order_by("-added")
    if request.user.is_authenticated():
        user_bookmarks = Bookmark.on_site.filter(saved_instances__user=request.user)
    else:
        user_bookmarks = []
    return render_to_response(template_name, {
        "bookmarks": bookmarks,
        "user_bookmarks": user_bookmarks,
    }, context_instance=RequestContext(request))


@login_required
def your_bookmarks(request, template_name="bookmarks/your_bookmarks.html"):
    bookmark_instances = BookmarkInstance.on_site.filter(user=request.user).order_by("-saved")
    return render_to_response(template_name, {
        "bookmark_instances": bookmark_instances,
    }, context_instance=RequestContext(request))

@login_required
def add(request, form_class=BookmarkInstanceForm,
        template_name="bookmarks/add.html"):

    if request.method == "POST":
        bookmark_form = form_class(request.user, request.POST)
        if bookmark_form.is_valid():
            bookmark_instance = bookmark_form.save(commit=False)
            bookmark_instance.user = request.user
            bookmark_instance.save()
            bookmark = bookmark_instance.bookmark

            try:
                headers = {
                    "Accept" : "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
                    "Accept-Language" : "en-us,en;q=0.5",
                    "Accept-Charset": "ISO-8859-1,utf-8;q=0.7,*;q=0.7",
                    "Connection" : "close",
                    ##"User-Agent": settings.URL_VALIDATOR_USER_AGENT
                    }
                req = urllib2.Request(bookmark.get_favicon_url(force=True), None, headers)
                u = urllib2.urlopen(req)
                has_favicon = True
            except:
                has_favicon = False

            bookmark.has_favicon = has_favicon
            bookmark.favicon_checked = datetime.now()
            bookmark.save()

            if bookmark_form.should_redirect():
                return HttpResponseRedirect(bookmark.url)
            else:
                messages.info(request, _("You have saved bookmark '{0}'".format(bookmark_instance.description)))
                return HttpResponseRedirect(reverse("bookmarks.views.bookmarks"))
    else:
        initial = {}
        if "url" in request.GET:
            initial["url"] = request.GET["url"]
        if "description" in request.GET:
            initial["description"] = request.GET["description"].strip()
        if "redirect" in request.GET:
            initial["redirect"] = request.GET["redirect"]

        if initial:
            bookmark_form = form_class(initial=initial)
        else:
            bookmark_form = form_class()

    bookmarks_add_url = "http://" + Site.objects.get_current().domain + reverse(add)
    bookmarklet = "javascript:location.href='%s?url='+encodeURIComponent(location.href)+';description='+encodeURIComponent(document.title)+';redirect=on'" % bookmarks_add_url

    return render_to_response(template_name, {
        "bookmarklet": bookmarklet,
        "bookmark_form": bookmark_form,
    }, context_instance=RequestContext(request))

@login_required
def delete(request, bookmark_instance_id):

    bookmark_instance = get_object_or_404(BookmarkInstance.on_site.all(), id=bookmark_instance_id)
    if request.user == bookmark_instance.user:
        #BookmarkInstance.objects.get(pk=bookmark_instance_id, user=request.user).delete()
        bookmark_instance.delete()
        messages.info(request, _("You have deleted bookmark '{0}'".format(bookmark_instance.description)))

    if "next" in request.GET:
        next = request.GET["next"]
    else:
        next = reverse("bookmarks.views.bookmarks")

    return HttpResponseRedirect(next)

